classdef dipoleAndMeasureOfStudy < pr.dipoleAndMeasure & pr.studyDipole
    % holds a measure (e.g. ERP, ERSP...) from one or several conditions in a linearized way.
    % the first dimension is the linearized version of the measure for concatenated subset of
    % consitions.
    % the second dimension of value is dipoles.
    properties
        conditionId;
        conditionLabel;
        relationshipToBrainVolume = 'insidebrain';
        scalpmap % information about dipole scalp maps
    end % properties       
    methods (Access = 'private')
        function measure2DWithConditionsInCells = getSeparatedConditionsForLinearized2DMeasure(obj, linearMeasureForCombinedCondition)
            % changce 1-d lineairzed measure into 2D
            numberOfTimeFramesInEachCondition = length(obj.time);
            measureForCombinedCondition = reshape(linearMeasureForCombinedCondition, [], numberOfTimeFramesInEachCondition * length(obj.conditionId))';
            
            % separate conditions (they are concatenated in the time dimension).
            measureWithConditionsInCells = {};
            for coditionNumber = obj.conditionId
                measure2DWithConditionsInCells{coditionNumber} = measureForCombinedCondition((1+(coditionNumber-1) * numberOfTimeFramesInEachCondition) : (numberOfTimeFramesInEachCondition * coditionNumber),:);
            end;
        end;
        function measure1DWithConditionsInCells = getSeparatedConditionsForLinearized1DMeasure(obj, linearMeasureForCombinedCondition)
            
            % usually conditions are concatenated in the time dimension, with the exception of spec
            % measure in which they are concatenated in the frequency dimension.
            if isfield(struct(obj), 'time')
                numberOfTimeFramesInEachCondition = length(obj.time);
            else
                numberOfTimeFramesInEachCondition = length(obj.frequency);
            end;
            
            % separate conditions (they are concatenated in the time dimension).
            measureWithConditionsInCells = {};
            for coditionNumber = obj.conditionId
                measure1DWithConditionsInCells{coditionNumber} = linearMeasureForCombinedCondition((1+(coditionNumber-1) * numberOfTimeFramesInEachCondition) : (numberOfTimeFramesInEachCondition * coditionNumber),:);
            end;
        end;
    end;
    methods (Abstract = true, Access = 'protected')
        plotMeasureAsArrayInCell(measureAsArrayInCell, title, varargin); % will be defined in subclasses (child classes).
    end;
    methods
        
        function obj = dipoleAndMeasureOfStudy(STUDY, ALLEEG, varargin) % constructor
            % conditionId is an optional input that specifies which conditions should be included.
            % by default all conditions are included.
            
            obj = obj@pr.dipoleAndMeasure();
            obj = obj@pr.studyDipole(STUDY, ALLEEG);
            
            if nargin > 0
                
                inputOptions = finputcheck(varargin, ...
                    { 'condition'         {'string' 'cell'}  []                       ''; ...
                    'location'       'string'     {'insidebrain' 'outsidebrain' 'any'}   'insidebrain';...
                    'loadScalpmap'   'string'    {'on' 'off'}    'on'   });
                
                
                % inputOptions = obj.checkInputOption(varargin);
                obj.relationshipToBrainVolume = inputOptions.location;
                
                if isempty(inputOptions.condition)
                    conditionId = [];
                elseif ischar(inputOptions.condition) % is condition is provided as a single string, like 'target'
                    conditionId = find(strcmp(STUDY.condition, inputOptions.condition));
                elseif iscell(inputOptions.condition) % is condition is provided as a cell, like {'target' 'nontarget'}
                    for i=1:length(inputOptions.condition)
                        conditionId(i) = find(strcmp(STUDY.condition, inputOptions.condition{i}));
                    end;
                end;
                
                if nargin < 3 || isempty(conditionId)
                    conditionId = 1:length(STUDY.condition);
                end
                
                obj.conditionId  = conditionId;
                obj.conditionLabel = STUDY.condition(conditionId);
                
                % read scalpmaps, mostly for visualization but also for correct ERP polarity.
                if strcmp(inputOptions.loadScalpmap, 'on')
                    obj.scalpmap = pr.scalpmapOfStudy(STUDY, ALLEEG, obj.icIndexForEachDipole);
                end;
            end;
        end;
        function newObj = createSubsetForId(obj, subsetId)
            newObj = obj;
            newObj.linearizedMeasure = obj.linearizedMeasure(:, subsetId);
            newObj.location = obj.location(subsetId,:);
            newObj.direction = obj.direction(subsetId,:);
            newObj.insideBrain = obj.insideBrain(subsetId);
            newObj.datasetId = obj.datasetId(subsetId);
            newObj.subjectName = obj.subjectName(subsetId);
            newObj.groupName = obj.groupName(subsetId);
            newObj.groupNumber = obj.groupNumber(subsetId);
            newObj.subjectNumber = obj.subjectNumber(subsetId);
            newObj.datasetIdAllConditions = obj.datasetIdAllConditions(:,subsetId);
            newObj.numberInDataset = obj.numberInDataset(subsetId);
            newObj.uniqueSubjectName = unique(newObj.subjectName);
            newObj.uniqueGroupName = unique(newObj.groupName);
            
            if ~isempty(newObj.scalpmap)
                newObj.scalpmap = obj.scalpmap.createSubsetForId(subsetId);
            end;
        end
        function [newObj subsetId]= createSubsetForGroup(obj, varargin)
            % get ids for the dipoles that belong to subset
            subsetId = getSubsetIdForGroup(obj, varargin{:});
            newObj =  createSubsetForId(obj, subsetId);
        end
        function [newObj subsetId] = createSubsetForSubject(obj, varargin)
            % get ids for the dipoles that belong to subset
            subsetId = getSubsetIdForSubject(obj, varargin{:});
            newObj =  createSubsetForId(obj, subsetId);
        end
        function [newObj subsetId] = createSubsetInRelationToBrain(obj, varargin)
            % get ids for the dipoles that belong to subset
            subsetId = getSubsetIdInRelationToBrain(obj, varargin{:});
            newObj =  createSubsetForId(obj, subsetId);
        end
        function measureWithConditionsInCells = getSeparatedConditionsForLinearizedMeasure(obj, linearMeasureForCombinedCondition)
            if obj.numberOfMeasureDimensions == 1
                measureWithConditionsInCells = getSeparatedConditionsForLinearized1DMeasure(obj, linearMeasureForCombinedCondition);
            elseif obj.numberOfMeasureDimensions == 2
                measureWithConditionsInCells = getSeparatedConditionsForLinearized2DMeasure(obj, linearMeasureForCombinedCondition);
            else
                error('Number of of measure dimensions is not assigned in ''numberOfMeasureDimensions'' property of dipoleAndMeasureOfStudy type object\n');
            end;
        end;
        function [linearProjectedMeasure groupId uniqeDatasetId] = getProjectedMeasureForEachSession(obj, position, standardDeviationOfEstimatedDipoleLocation)
            % [linearProjectedMeasure uniqeDatasetId groupId] = getProjectedMeasureForEachSession(obj, position, standardDeviationOfEstimatedDipoleLocation)
            %
            % projects each session (dataset) to provided position(s) and returns a NxPxS matrix.
            % N is the numbr of dimensions of the linearized measure.
            % P is the number of points in the input position parameter.
            % S is the number of sessions (datasets)
            
            % go through all sessions and project session dipoles to the given location(s)
            uniqeDatasetId = unique(obj.datasetId);
            linearProjectedMeasure = [];
            groupId = []; % hold group ids (numbers) for datasets
            for datasetId = uniqeDatasetId
                dipoleAndMeasureForDataset = obj.createSubsetForId(find(obj.datasetId == datasetId));
                projectionFromTheDataset = dipoleAndMeasureForDataset.projectTo(position, standardDeviationOfEstimatedDipoleLocation);
                linearProjectedMeasure = cat(3, linearProjectedMeasure, projectionFromTheDataset);
                groupId = [groupId  dipoleAndMeasureForDataset.groupNumber(1)]; % get the group number of dataset
            end;
        end;
        function [differenceBetweenConditions significanceValue] = getMeasureDifferenceAcrossConditions(obj, position, standardDeviationOfEstimatedDipoleLocation, twoConditionIdsForComparison, usePositionProjections, varargin)
            % [differenceBetweenConditions significanceValue] = getMeasureDifferenceAcrossConditions(obj, position, standardDeviationOfEstimatedDipoleLocation, twoConditionIdsForComparison, usePositionProjections, [optional statcond() statistics parameters])
            %
            % usePositionProjections specifies how different projected measures for different positions are used.
            % It can be 'each' to indicate using all projections for each session as independent
            % measurement (for example if they are from a Domain, if we have 20 positions in the domain 
            % and 10 subjects, there will be 20*10 = 200 measurements passed to statcond). 
            % The other choice for usePositionProjectionsis is 'mean', in which case only
            % the average of the measure over given positions will be used from each session.
            
            
            % first we project each dataset separately to the given location, then we separate
            % projected values for each condition and calculate statitics
            
            % by default compare first and second conditions
            if nargin < 4
                twoConditionIdsForComparison = [1 2];
            end;
            
            % by default use each projected position for each session as an independent measurement.
            if nargin < 5 || isempty(usePositionProjections)
                usePositionProjections = 'each';
            end;
            
            % go through all sessions and project session dipoles to the given position(s)
            linearProjectedMeasure = obj.getProjectedMeasureForEachSession(position, standardDeviationOfEstimatedDipoleLocation);
            
            if strcmpi(usePositionProjections, 'each')
                % add all projected measures from all subjects together, Multiple position for each
                % subject will contribute.
                linearProjectedMeasureForCombinedCondition = reshape(linearProjectedMeasure, size(linearProjectedMeasure,1), []);                
            else % when it is 'mean'
                % we average the linearized measure over all the given position, which are in the second
                % dimension.
                linearProjectedMeasureForCombinedCondition = squeeze(mean(linearProjectedMeasure,2));                
            end;

            for coditionNumber = obj.conditionId
                projectedMeasure{coditionNumber} = [];
            end;
            
            for datasetCounter = 1:size(linearProjectedMeasureForCombinedCondition,2)
                projectedMeasureForDatasetWithConditionsSeparated = obj.getSeparatedConditionsForLinearizedMeasure(linearProjectedMeasureForCombinedCondition(:,datasetCounter));
                
                % separate conditions (they are concatenated in the time dimension).
                for coditionNumber = obj.conditionId
                    projectedMeasure{coditionNumber} = cat(3, projectedMeasure{coditionNumber}, projectedMeasureForDatasetWithConditionsSeparated{coditionNumber});
                end;
            end;
            
            % default statiscal method
            %             if isempty(varargin)
            %                  varargin = {'mode', 'perm', 'naccu', 250};
            %             end;
            
            % calculate statistical significance
            [differenceBetweenConditions degreesOfFreedom significanceValue] = statcond(projectedMeasure(twoConditionIdsForComparison), varargin{:});
        end;
        function [groupConditionMean differenceBetweenGroups significanceValue] = getMeasureDifferenceAcrossGroups(obj, position, standardDeviationOfEstimatedDipoleLocation, twoGroupIdsForComparison, conditionToCompareId, usePositionProjections, varargin)
            % [differenceBetweenGroups significanceValue] = getMeasureDifferenceAcrossGroups(obj, position, standardDeviationOfEstimatedDipoleLocation, twoGroupIdsForComparison, conditionId, [optional statcond() statistics parameters])
            
            % first we project each dataset separately to the given location, then we separate
            % projected values for the condition of interest and calculate statiticsfor given groups
            
            % by default compare first and second group
            if nargin < 4
                twoGroupIdsForComparison = [1 2];
            end;
                    
            % by default compare the first condition across groups
            if nargin < 5
                twoConditionIdsForComparison = 1;
            end;
            
            % by default use each projected position for each session as an independent measurement.
            if nargin < 6 || isempty(usePositionProjections)
                usePositionProjections = 'mean';
            end;
            
            % go through all sessions and project session dipoles to the given position(s)
            [linearProjectedMeasure datasetGroupNumber] = obj.getProjectedMeasureForEachSession(position, standardDeviationOfEstimatedDipoleLocation);
            
            if strcmpi(usePositionProjections, 'each')
                % add all projected measures from all subjects together, Multiple position for each
                % subject will contribute.
                linearProjectedMeasureForCombinedCondition = reshape(linearProjectedMeasure, size(linearProjectedMeasure,1), []);
                datasetGroupNumber = repmat(datasetGroupNumber, size(linearProjectedMeasure,2),1);
                datasetGroupNumber =  reshape(datasetGroupNumber, 1, []);
            else % when usePositionProjections = 'mean'
                % average the linearized measure over all the given position, which are in the second
                % dimension.
                linearProjectedMeasureForCombinedCondition = squeeze(mean(linearProjectedMeasure,2));
            end;
            
            for groupNumber = 1:obj.numberOfGroups
                projectedMeasure{groupNumber} = [];
            end;
            
            for datasetCounter = 1:size(linearProjectedMeasureForCombinedCondition,2)
                projectedMeasureForDatasetWithConditionsSeparated = obj.getSeparatedConditionsForLinearizedMeasure(linearProjectedMeasureForCombinedCondition(:,datasetCounter));
                
                % separate conditions (they are concatenated in the time dimension).
                for coditionNumber = obj.conditionId
                    projectedMeasure{datasetGroupNumber(datasetCounter)} = cat(3, projectedMeasure{datasetGroupNumber(datasetCounter)}, projectedMeasureForDatasetWithConditionsSeparated{conditionToCompareId});
                end;
            end;
            
            % calculate group-mean for the condition and place them in a cell-array.
            for i=1:2
                groupConditionMean{i} = mean(projectedMeasure{twoGroupIdsForComparison(i)},3);
            end;
            
            % default statiscal method
            %  if isempty(varargin)
            %       varargin = {'mode', 'perm', 'naccu', 250};
            %  end;
            
            % calculate statistical significance
            [differenceBetweenGroups degreesOfFreedom significanceValue] = statcond(projectedMeasure(twoGroupIdsForComparison), varargin{:});
        end;
        function plot(obj, linearProjectedMeasureForCombinedCondition, varargin)
            % varargin is for passing parameters to the plotting function
            
            projectedMeasure = obj.getSeparatedConditionsForLinearizedMeasure(linearProjectedMeasureForCombinedCondition);
            obj.plotMeasureAsArrayInCell(projectedMeasure, obj.conditionLabel, varargin{:});
        end;
        function plotConditionDifference(obj, linearProjectedMeasureForCombinedCondition, position, standardDeviationOfEstimatedDipoleLocation, twoConditionLabelsForComparison, significanceLevelForConditionDifference, usePositionProjections, statisticsParameter, plottingParameter)
            % plotConditionDifference(linearProjectedMeasureForCombinedCondition, position, standardDeviationOfEstimatedDipoleLocation, twoConditionLabelsForComparison, significanceLevelForConditionDifference, statisticsParameter, plottingParameter)
            %
            % plot condition difference between first and second specified conditions (A-B) and
            % mask areas with non-significent differences.
            
            % if no condition label is provided, use the first two (if they exist)
            if nargin < 5 || isempty(twoConditionLabelsForComparison)
                if length(obj.conditionLabel) > 1
                    twoConditionIdsForComparison = [1 2];
                    twoConditionLabelsForComparison = obj.conditionLabel;
                    
                    % when there are more than two conditions present, give a warning
                    if length(obj.conditionLabel) > 2
                        fprintf('Warning: there are more than two conditons present, by default only the difference between the first two is displayed. You can assign another condition pair in the input.\n');
                    end;
                else
                    error('There is only one condition present, at least two conditions are needed for a comparison\n');
                end;
            else % if condition labels for comparison are provided, then find condition ids in obj.conditionLabel that are associated with them.                
                if length(twoConditionLabelsForComparison) < 2
                    error('Number of provided condition labels is less than required two (2).\n');
                end;
                
                % find condition ids in obj.conditionLabel that are associated with provided conditon labels.
                for i=1:2
                    twoConditionIdsForComparison(i) = find(strcmp(obj.conditionLabel, twoConditionLabelsForComparison{i}));
                end;
            end
            
            if nargin < 6
                significanceLevelForConditionDifference = 0.03;                
            end;
            
            if nargin < 7
                usePositionProjections = 'each';
            end;
            
            if nargin < 8               
                statisticsParameter = {};
            end;
            
            if nargin < 9               
                plottingParameter = {};
            end;
            
            
            % separate conditions from linearized form
            projectedMeasure = obj.getSeparatedConditionsForLinearizedMeasure(linearProjectedMeasureForCombinedCondition);
            
            % only keep the two conditions to be compared
            projectedMeasure = projectedMeasure(twoConditionIdsForComparison);
            
            % calculate difference statistics for two conditions
            [differenceBetweenConditions significanceValue] = obj.getMeasureDifferenceAcrossConditions(position, standardDeviationOfEstimatedDipoleLocation, twoConditionIdsForComparison, usePositionProjections,statisticsParameter{:});
            
            % calculate difference between the two conditions and add as the third measure for
            % plotting
            projectedMeasure{3} = projectedMeasure{1} - projectedMeasure{2};
            
            % mask insignificent differences
            projectedMeasure{3}(significanceValue > significanceLevelForConditionDifference) = 0;
            
            % set the titles
            conditionTitle = [obj.conditionLabel{twoConditionIdsForComparison(1)} ' - ' obj.conditionLabel{twoConditionIdsForComparison(2)} ' (p<' num2str(significanceLevelForConditionDifference) ')'];
            
            title = [obj.conditionLabel(twoConditionIdsForComparison), conditionTitle];
            
            % plot
            obj.plotMeasureAsArrayInCell(projectedMeasure, title, plottingParameter{:});
        end;
        function plotGroupDifference(obj, linearProjectedMeasureForCombinedCondition, position, standardDeviationOfEstimatedDipoleLocation, twoGroupLabelsForComparison, conditionLabelForComparison, significanceLevelForConditionDifference, usePositionProjections, statisticsParameter, plottingParameter)
            % plotGroupDifference(obj, linearProjectedMeasureForCombinedCondition, position, standardDeviationOfEstimatedDipoleLocation, twoGroupLabelsForComparison, conditionLabelsForComparison, significanceLevelForConditionDifference, statisticsParameter, plottingParameter)
            %
            % plot group projected measures and their difference for a given condition. The difference is calculated between the first and second specified group (A-B) 
            % Areas with non-significent differences are masked.
            
            % if no group label is provided, use the first two (if they exist)
            if nargin < 5 || isempty(twoGroupLabelsForComparison)
                if obj.numberOfGroups  > 1
                    twoGroupIdsForComparison = [1 2];
                    twoGroupLabelsForComparison = obj.uniqueGroupName(twoGroupIdsForComparison);
                    
                    % when there are more than two conditions present, give a warning
                    if obj.numberOfGroups > 2
                        fprintf('Warning: there are more than two groups present, by default only the difference between the first two is displayed. You can assign another group pair in the input.\n');
                    end;
                else
                    error('There is only one group present, at least two groups are needed for a comparison\n');
                end;
            else % if group labels for comparison are provided, then find group ids (numbers) in obj.uniqueGroupName that are associated with them.                
                if length(twoGroupLabelsForComparison) < 2
                    error('Number of provided condition labels is less than required two (2).\n');
                end;
                
                % find group ids in obj.uniqueGroupName that are associated with provided group labels.
                for i=1:2
                    twoGroupIdsForComparison(i) = find(strcmp(obj.uniqueGroupName, twoGroupLabelsForComparison{i}));
                end;
            end
            
            % if no condition label was specified, by default use the first condition
            if nargin < 6 || isempty(conditionLabelForComparison)
                conditionToCompareId = 1;
                conditionLabelForComparison = obj.conditionLabel{conditionToCompareId};
            else % othrwise find the condition id associated with the provided condition label.
                conditionToCompareId = find(strcmp(obj.conditionLabel, conditionLabelForComparison));                
            end;
            
            if nargin < 7
                significanceLevelForConditionDifference = 0.03;                
            end;
            
            if nargin < 8
                usePositionProjections = 'mean';
            end;
            
            if nargin < 9               
                statisticsParameter = {};
            end;
            
            if nargin < 10               
                plottingParameter = {};
            end;
            
            % calculate difference statistics for two groups on one condition
            [groupConditionMean differenceBetweenGroups significanceValue] = obj.getMeasureDifferenceAcrossGroups(position, standardDeviationOfEstimatedDipoleLocation, twoGroupIdsForComparison, conditionToCompareId, usePositionProjections, statisticsParameter{:});            
            projectedMeasure = groupConditionMean;
                       
            % calculate difference between the two groups and add it as the third measure for
            % plotting
            projectedMeasure{3} = projectedMeasure{1} - projectedMeasure{2};
            
            % mask insignificent differences
            projectedMeasure{3}(significanceValue > significanceLevelForConditionDifference) = 0;
            
            % set the titles
            groupDifferenceTitle = ['Condition ' conditionLabelForComparison ', Group ' twoGroupLabelsForComparison{1} ' - ' twoGroupLabelsForComparison{2} ' (p<' num2str(significanceLevelForConditionDifference) ')'];
            
            title = [twoGroupLabelsForComparison, groupDifferenceTitle];
            
            % plot
            obj.plotMeasureAsArrayInCell(projectedMeasure, title, plottingParameter{:});
        end
    end    
end